import { EVENTS, EVENTHANDLE_INPUT_TIMEOUT } from "../Constants";
import { noValue } from "@default-js/defaultjs-common-utils/src/ValueHelper";
import { toTimeoutHandle } from "../utils/EventHelper";
import Wrapper from "./Wrapper";

const INPUTSELECTOR = 'input:not([type="file"]):not([type="radio"]):not([type="checkbox"]) ,input:not([type]), textarea';

const DEFAULTTYPE = "text";

const text = {
	accept: (value) => {
		return typeof value === "string";
	},
	value: (input) => {
		return input.value;
	},
	normalize: (value) => {
		if (value) {
			value = value.trim();
			return value.length > 0 ? value : null;
		}

		return null;
	},
};
const number = {
	accept: (value) => {
		return typeof value === "number";
	},
	value: (input) => {
		return input.valueAsNumber;
	},
	normalize: (value) => {
		if (!noValue(value) && !Number.isNaN(value)) return value;

		return null;
	},
};
const date = {
	accept: (value) => {
		return value instanceof Date;
	},
	value: (input) => {
		return input.valueAsDate;
	},
	normalize: (value) => {
		if (value) return value;

		return null;
	},
};
const TYPES = { text, number, date, time: date };

export default class Text extends Wrapper {
	static findInput(field) {
		return field.find(INPUTSELECTOR).first();
	}

	constructor(field, input) {
		super(field, input);
	}

	init() {
		const { field, input } = this;
		const type = (field.attr("input-type") || input.attr("type") || DEFAULTTYPE).trim().toLowerCase();
		this.type = TYPES[type] || TYPES[DEFAULTTYPE];
		input.on(
			"input",
			toTimeoutHandle(
				() => {
					field.trigger(EVENTS.input, this.normalizeValue(this.value));
				},
				false,
				true,
			),
		);

		field.trigger(EVENTS.input, this.normalizeValue(this.value));
	}

	acceptValue(value) {
		if (value == null || typeof value === "undefined") return true;

		return this.type.accept(value);
	}

	normalizeValue(value) {
		if (value == null && typeof value === "undefined") return null;

		return this.type.normalize(value);
	}
	async updatedValue(value) {
		if (value != this.input.value)
			this.input.val(value ? value : null);
	}

	set readonly(readonly) {
		this.input.attr("disabled", readonly ? "" : null);
	}

	get value() {
		return this.type.value(this.input);
	}

	get valid() {
		return this.input.checkValidity();
	}
}
